@@ -16,46 +16,41 @@
 #' @param url the character host or dispatcher URL to dial into, including the
 #'   port to connect to, e.g. 'tcp://hostname:5555' or
 #'   'tls+tcp://10.75.32.70:5555'.
-#' @param dispatcher \[default TRUE\] logical value, which should be set to
-#'   TRUE if using dispatcher and FALSE otherwise.
+#' @param dispatcher logical value, which should be set to TRUE if using
+#'   dispatcher and FALSE otherwise.
 #' @param ... reserved, but not currently used.
-#' @param asyncdial \[default FALSE\] whether to perform dials asynchronously.
-#'   The default FALSE will error if a connection is not immediately possible
-#'   (for instance if [daemons()] has yet to be called on the host, or the
-#'   specified port is not open etc.). Specifying TRUE continues retrying
-#'   (indefinitely) if not immediately successful, which is more resilient but
-#'   can mask potential connection issues.
-#' @param autoexit \[default TRUE\] logical value, whether the daemon should
-#'   exit automatically when its socket connection ends. By default, the process
-#'   ends immediately when the host process ends. Supply `NA` to have a daemon
-#'   complete any tasks in progress before exiting (see 'Persistence' section
-#'   below).
-#' @param cleanup \[default TRUE\] logical value, whether to perform cleanup of
-#'   the global environment and restore attached packages and options to an
-#'   initial state after each evaluation.
-#' @param output \[default FALSE\] logical value, to output generated stdout /
-#'   stderr if TRUE, or else discard if FALSE. Specify as TRUE in the `...`
-#'   argument to [daemons()] or [launch_local()] to provide redirection of
-#'   output to the host process (applicable only for local daemons).
-#' @param idletime \[default Inf\] integer milliseconds maximum time to wait for
-#'   a task (idle time) before exiting.
-#' @param walltime \[default Inf\] integer milliseconds soft walltime (time
-#'   limit) i.e. the minimum amount of real time elapsed before exiting.
-#' @param maxtasks \[default Inf\] integer maximum number of tasks to execute
-#'   (task limit) before exiting.
-#' @param tlscert \[default NULL\] required for secure TLS connections over
-#'   'tls+tcp://'. **Either** the character path to a file containing X.509
-#'   certificate(s) in PEM format, comprising the certificate authority
-#'   certificate chain starting with the TLS certificate and ending with the CA
-#'   certificate, **or** a length 2 character vector comprising \[i\] the
-#'   certificate authority certificate chain and \[ii\] the empty string `""`.
-#' @param tls deprecated, please use `tlscert` instead.
-#' @param id \[default NULL\] (optional) integer daemon ID provided to
-#'   dispatcher to track connection status. Causes [status()] to report this ID
-#'   under `$events` when the daemon connects and disconnects.
-#' @param rs \[default NULL\] the initial value of .Random.seed. This is set
-#'   automatically using L'Ecuyer-CMRG RNG streams generated by the host process
-#'   if applicable, and should not be independently supplied.
+#' @param asyncdial whether to perform dials asynchronously. The default FALSE
+#'   will error if a connection is not immediately possible (for instance if
+#'   [daemons()] has yet to be called on the host, or the specified port is not
+#'   open etc.). Specifying TRUE continues retrying (indefinitely) if not
+#'   immediately successful, which is more resilient but can mask potential
+#'   connection issues.
+#' @param autoexit logical value, whether the daemon should exit automatically
+#'   when its socket connection ends. By default, the process ends immediately
+#'   when the host process ends. Supply `NA` to have a daemon complete any tasks
+#'   in progress before exiting (see 'Persistence' section below).
+#' @param cleanup logical value, whether to perform cleanup of the global
+#'   environment and restore attached packages and options to an initial state
+#'   after each evaluation.
+#' @param output logical value, to output generated stdout / stderr if TRUE, or
+#'   else discard if FALSE. Specify as TRUE in the `...` argument to [daemons()]
+#'   or [launch_local()] to provide redirection of output to the host process
+#'   (applicable only for local daemons).
+#' @param idletime integer milliseconds maximum time to wait for a task (idle
+#'   time) before exiting.
+#' @param walltime integer milliseconds soft walltime (time limit) i.e. the
+#'   minimum amount of real time elapsed before exiting.
+#' @param maxtasks integer maximum number of tasks to execute (task limit)
+#'   before exiting.
+#' @param tlscert required for secure TLS connections over 'tls+tcp://'.
+#'   **Either** the character path to a file containing X.509 certificate(s) in
+#'   PEM format, comprising the certificate authority certificate chain starting
+#'   with the TLS certificate and ending with the CA certificate, **or** a
+#'   length 2 character vector comprising (i) the certificate authority
+#'   certificate chain and (ii) the empty string `""`.
+#' @param rs the initial value of .Random.seed. This is set automatically using
+#'   L'Ecuyer-CMRG RNG streams generated by the host process if applicable, and
+#'   should not be independently supplied.
 #'
 #' @return Invisibly, an integer exit code: 0L for normal termination, and a
 #'   positive value if a self-imposed limit was reached: 1L (idletime), 2L
@@ -93,8 +88,6 @@ daemon <- function(
   walltime = Inf,
   maxtasks = Inf,
   tlscert = NULL,
-  tls = tlscert,
-  id = NULL,
   rs = NULL
 ) {
   cv <- cv()
@@ -105,8 +98,8 @@ daemon <- function(
   })
   `[[<-`(., "sock", sock)
   pipe_notify(sock, cv, remove = TRUE, flag = flag_value_auto(autoexit))
-  if (length(tls)) tls <- tls_config(client = tls)
-  dial_sync_socket(sock, url, autostart = asyncdial || NA, tls = tls)
+  if (length(tlscert)) tlscert <- tls_config(client = tlscert)
+  dial_sync_socket(sock, url, autostart = asyncdial || NA, tls = tlscert)
 
   if (!output) {
     devnull <- file(nullfile(), open = "w", blocking = FALSE)
@@ -117,44 +110,39 @@ daemon <- function(
   task <- 1L
   timeout <- if (idletime > walltime) walltime else if (is.finite(idletime)) idletime
   maxtime <- if (is.finite(walltime)) mclock() + walltime else FALSE
-  if (otel_tracing) {
-    spn <- otel::start_local_active_span(
-      "mirai::daemon",
-      attributes = otel::as_attributes(list(url = url))
-    )
-  }
+  if (otel_tracing) dmnspn <- otel_daemon_span(url)
 
   if (dispatcher) {
     aio <- recv_aio(sock, mode = 1L, cv = cv)
-    is.numeric(id) && send(sock, c(0L, as.integer(id)), mode = 2L, block = TRUE)
-    wait(cv) || return(invisible(xc))
-    bundle <- collect_aio(aio)
-    `[[<-`(.GlobalEnv, ".Random.seed", if (is.numeric(rs)) as.integer(rs) else bundle[[1L]])
-    if (is.list(bundle[[2L]])) `opt<-`(sock, "serial", bundle[[2L]])
-    snapshot()
-    repeat {
-      aio <- recv_aio(sock, mode = 1L, timeout = timeout, cv = cv)
-      wait(cv) || break
-      m <- collect_aio(aio)
-      is.integer(m) && {
-        m == 5L || next
-        xc <- 1L
-        break
+    if (wait(cv)) {
+      bundle <- collect_aio(aio)
+      `[[<-`(globalenv(), ".Random.seed", if (is.numeric(rs)) as.integer(rs) else bundle[[1L]])
+      if (is.list(bundle[[2L]])) `opt<-`(sock, "serial", bundle[[2L]])
+      snapshot()
+      repeat {
+        aio <- recv_aio(sock, mode = 1L, timeout = timeout, cv = cv)
+        wait(cv) || break
+        m <- collect_aio(aio)
+        is.integer(m) && {
+          m == 5L || next
+          xc <- 1L
+          break
+        }
+        (task >= maxtasks || maxtime && mclock() >= maxtime) && {
+          .mark()
+          send(sock, eval_mirai(m, sock), mode = 1L, block = TRUE)
+          aio <- recv_aio(sock, mode = 8L, cv = cv)
+          xc <- 2L + (task >= maxtasks)
+          wait(cv)
+          break
+        }
+        send(sock, eval_mirai(m, sock), mode = 1L, block = TRUE)
+        if (cleanup) do_cleanup()
+        task <- task + 1L
       }
-      (task >= maxtasks || maxtime && mclock() >= maxtime) && {
-        .mark()
-        send(sock, eval_mirai_with_cancel(m, sock), mode = 1L, block = TRUE)
-        aio <- recv_aio(sock, mode = 8L, cv = cv)
-        xc <- 2L + (task >= maxtasks)
-        wait(cv)
-        break
-      }
-      send(sock, eval_mirai_with_cancel(m, sock), mode = 1L, block = TRUE)
-      if (cleanup) do_cleanup()
-      task <- task + 1L
     }
   } else {
-    if (is.numeric(rs)) `[[<-`(.GlobalEnv, ".Random.seed", as.integer(rs))
+    if (is.numeric(rs)) `[[<-`(globalenv(), ".Random.seed", as.integer(rs))
     snapshot()
     repeat {
       ctx <- .context(sock)
@@ -165,16 +153,20 @@ daemon <- function(
         xc <- 1L
         break
       }
-      send(ctx, eval_mirai(m), mode = 1L, block = TRUE)
-      if (cleanup) do_cleanup()
       (task >= maxtasks || maxtime && mclock() >= maxtime) && {
+        .mark()
+        send(ctx, eval_mirai(m), mode = 1L, block = TRUE)
         xc <- 2L + (task >= maxtasks)
+        wait(cv)
         break
       }
+      send(ctx, eval_mirai(m), mode = 1L, block = TRUE)
+      if (cleanup) do_cleanup()
       task <- task + 1L
     }
   }
 
+  if (otel_tracing) otel_daemon_span(url, end_span = dmnspn)
   if (!output) {
     sink(type = "message")
     sink()
@@ -201,8 +193,9 @@ daemon <- function(
   pipe_notify(sock, cv, remove = TRUE, flag = flag_value())
   dial(sock, url = url, autostart = NA, fail = 2L)
   `[[<-`(., "sock", sock)
+  .mark()
   m <- recv(sock, mode = 1L, block = TRUE)
-  send(sock, eval_mirai(m), mode = 1L, block = TRUE) || until(cv, .limit_short)
+  send(sock, eval_mirai(m), mode = 1L, block = TRUE) || wait(cv)
 }
 
 # internals --------------------------------------------------------------------
@@ -217,20 +210,25 @@ handle_mirai_interrupt <- function(cnd) {
   invokeRestart("mirai_interrupt")
 }
 
-eval_mirai <- function(._mirai_.) {
+eval_mirai <- function(._mirai_., sock = NULL) {
   withRestarts(
     withCallingHandlers(
       {
-        list2env(._mirai_.[["._globals_."]], envir = .GlobalEnv)
+        if (length(sock)) {
+          cancel <- recv_aio(sock, mode = 8L, cv = substitute())
+          on.exit(stop_aio(cancel))
+        }
+        list2env(._mirai_.[["._globals_."]], envir = globalenv())
         if (otel_tracing && length(._mirai_.[["._otel_."]])) {
           prtctx <- otel::extract_http_context(._mirai_.[["._otel_."]])
           spn <- otel::start_local_active_span(
-            "mirai::daemon->eval",
-            links = list(daemon = otel::get_active_span()),
-            options = list(kind = "server", parent = prtctx)
+            "daemon->eval",
+            links = list(daemon = dynGet("dmnspn")),
+            options = list(kind = "server", parent = prtctx),
+            tracer = otel_tracer
           )
         }
-        eval(._mirai_.[["._expr_."]], envir = ._mirai_., enclos = .GlobalEnv)
+        eval(._mirai_.[["._expr_."]], envir = ._mirai_., enclos = globalenv())
       },
       error = handle_mirai_error,
       interrupt = handle_mirai_interrupt
@@ -240,10 +238,13 @@ eval_mirai <- function(._mirai_.) {
   )
 }
 
-eval_mirai_with_cancel <- function(._mirai_., sock) {
-  cancel <- recv_aio(sock, mode = 8L, cv = substitute())
-  on.exit(stop_aio(cancel))
-  eval_mirai(._mirai_.)
+otel_daemon_span <- function(url, end_span = NULL) {
+  otel::start_local_active_span(
+    if (length(end_span)) "daemon->end" else "daemon",
+    attributes = otel::as_attributes(list(url = url)),
+    links = if (length(end_span)) list(daemon = end_span),
+    tracer = otel_tracer
+  )
 }
 
 dial_sync_socket <- function(sock, url, autostart = NA, tls = NULL) {
@@ -255,14 +256,14 @@ dial_sync_socket <- function(sock, url, autostart = NA, tls = NULL) {
 }
 
 do_cleanup <- function() {
-  vars <- names(.GlobalEnv)
-  rm(list = vars[!vars %in% .[["vars"]]], envir = .GlobalEnv)
+  vars <- names(globalenv())
+  rm(list = vars[!vars %in% .[["vars"]]], envir = globalenv())
   new <- search()
   lapply(new[!new %in% .[["se"]]], detach, character.only = TRUE)
   options(.[["op"]])
 }
 
-snapshot <- function() `[[<-`(`[[<-`(`[[<-`(., "op", .Options), "se", search()), "vars", names(.GlobalEnv))
+snapshot <- function() `[[<-`(`[[<-`(`[[<-`(., "op", .Options), "se", search()), "vars", names(globalenv()))
 
 flag_value_auto <- function(autoexit) {
   (isFALSE(autoexit) || isNamespace(topenv(parent.frame(), NULL))) && return(autoexit) ||
