@@ -32,6 +32,7 @@ test_error(daemons(1, dispatcher = ""))
 test_error(daemons(url = local_url(), dispatcher = NA))
 test_error(daemon("URL"), "Invalid argument")
 test_error(launch_local(1L), "daemons must be set")
+test_error(race_mirai(list()), "daemons must be set")
 test_false(daemons_set())
 test_error(require_daemons())
 test_error(mirai:::stop_d(NULL), "No daemons set.")
@@ -55,7 +56,6 @@ test_true(mirai:::is.promising.mirai())
 test_true(mirai:::is.promising.mirai_map())
 test_error(everywhere({}))
 test_null(nextstream())
-test_null(nextget("pid"))
 for (i in 0:4)
   test_type("character", nextcode(i))
 test_null(register_serial("test_klass1", serialize, unserialize))
@@ -86,7 +86,7 @@ connection && {
   test_true(d <- daemons(1L, dispatcher = FALSE, asyncdial = FALSE, seed = 1546L))
   test_print(d)
   test_true(daemons_set())
-  test_true(require_daemons(parent.frame()))
+  test_true(suppressWarnings(require_daemons(parent.frame())))
   me <- mirai(mirai::mirai(), .timeout = 2000L)[]
   test_notnull(me)
   if (is_mirai_error(me)) test_type("list", me$stack.trace)
@@ -104,6 +104,12 @@ connection && {
   test_false(unresolved(dm))
   if (!is_error_value(dm$data)) test_class("matrix", dm$data)
   test_print(dm)
+  m1 <- mirai(Sys.sleep(0.1), .timeout = 200L)
+  m3 <- mirai({}, .timeout = 200L)
+  m2 <- mirai(Sys.sleep(0.1), .timeout = 200L)
+  test_type("list", race_mirai(list(m1, m2)))
+  test_type("list", race_mirai(list(m1, m2)))
+  test_type("list", race_mirai(list(m1, m2)))
   test_type("integer", info())
   test_type("integer", status()[["connections"]])
   test_type("character", status()[["daemons"]])
@@ -147,7 +153,7 @@ connection && {
   test_error(launch_remote(remote = ssh_config("ssh://127.0.0.1:5456", tunnel = TRUE)), "127.0.0.1")
   test_false(daemons(0L))
   Sys.sleep(1L)
-  test_true(daemons(n = 2L, url = value <- "ws://:0", dispatcher = "none", remote = remote_config(quote = TRUE)))
+  test_true(daemons(n = 2L, url = value <- "ws://:0", dispatcher = FALSE, remote = remote_config(quote = TRUE)))
   test_true(status()$daemons != value)
   test_false(daemons(0L))
 }
@@ -192,7 +198,7 @@ connection && {
   test_type("character", launch_remote(cluster[[1L]]))
   test_type("list", status(cluster))
   clusterSetRNGStream(cluster, 123)
-  j <- clusterEvalQ(cluster, expr = .GlobalEnv[[".Random.seed"]])
+  j <- clusterEvalQ(cluster, expr = globalenv()[[".Random.seed"]])
   a <- parSapply(cluster, 1:4, runif)
   setDefaultCluster(cluster)
   res <- parLapply(X = 1:10, fun = rnorm)
@@ -215,7 +221,7 @@ connection && {
   test_identical(parApply(cluster, df, 2, sum), `names<-`(c(6, 21), c("a", "b")))
   test_identical(parCapply(cluster, df, sum), `names<-`(c(6, 21), c("a", "b")))
   test_identical(parRapply(cluster, df, sum), `names<-`(c(7, 9, 11), c("1", "2", "3")))
-  res <- clusterEvalQ(expr = .GlobalEnv[[".Random.seed"]][[1L]])
+  res <- clusterEvalQ(expr = globalenv()[[".Random.seed"]][[1L]])
   test_type("integer", res[[1L]])
   test_error(clusterEvalQ(cluster, elephant()), "Error in elephant(): could not find function \"elephant\"")
   test_null(stop_cluster(cluster))
@@ -223,7 +229,7 @@ connection && {
   test_class("miraiCluster", cl <- make_cluster(1))
   test_true(attr(cl, "id") != attr(cluster, "id"))
   clusterSetRNGStream(cl, 123)
-  k <- clusterEvalQ(cl, expr = .GlobalEnv[[".Random.seed"]])
+  k <- clusterEvalQ(cl, expr = globalenv()[[".Random.seed"]])
   b <- parSapply(cl, 1:4, runif)
   test_identical(j, k)
   test_identical(a, b)
@@ -250,7 +256,7 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_false(daemons(0L))
   test_zero(with(daemons(url = "tcp://:0", correcttype = c(1, 0), token = TRUE), {8L - 9L + 1L}))
   test_true(daemons(n = 2, "ws://:0"))
-  test_type("integer", nextget("pid"))
+  test_type("character", nextget("dispatcher"))
   test_equal(length(nextget("url")), 1L)
   Sys.sleep(1L)
   status <- status()
@@ -273,15 +279,21 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_equal(launch_local(), 1L)
   Sys.sleep(1L)
   test_true(grepl("CERTIFICATE", launch_remote(), fixed = TRUE))
-  q <- quote(list2env(list(b = 2), envir = .GlobalEnv))
+  q <- quote(list2env(list(b = 2), envir = globalenv()))
   m <- mirai("Seattle", .timeout = 1000)
   if (!is_error_value(m[])) test_equal(m[], "Seattle")
   test_class("errorValue", mirai(q(), .timeout = 1000)[])
-  test_true(daemons(n = 1L, url = local_url(), dispatcher = TRUE))
-  task <- mirai(substitute())
-  url <- nextget("url")
-  test_equal(daemon(url = url, maxtasks = 1L, cleanup = 0L, dispatcher = TRUE), 3L)
-  test_false(daemons(n = 0L))
+  test_true(daemons(sync = TRUE, .compute = "seq"))
+  with_daemons("seq", {
+    everywhere({}, foo = "bar")
+    test_equal(mirai(foo)[], "bar")
+    test_error(launch_local(), "daemons cannot be launched")
+    test_error(launch_remote(), "synchronous compute profiles")
+    task <- mirai(substitute())
+  })
+  test_false(daemons(0, .compute = "seq"))
+  res <- task[]
+  test_true(missing(res))
   test_type("integer", .Random.seed)
 }
 # TLS tests
@@ -292,7 +304,7 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   if (.Platform$OS.type == "unix") test_type("character", launch_remote(remote = cluster_config(command = "/bin/sh", options = "#SBATCH", rscript = file.path(R.home("bin"), "Rscript"))))
   test_equal(launch_local(1L), 1L)
   Sys.sleep(1L)
-  q <- quote({ list2env(list(b = 2), envir = .GlobalEnv); 0L})
+  q <- quote({ list2env(list(b = 2), envir = globalenv()); 0L})
   mm <- everywhere(q)
   test_type("list", mm)
   test_zero(sum(collect_mirai(mm, ".flat")))
@@ -341,21 +353,19 @@ connection && requireNamespace("promises", quietly = TRUE) && Sys.getenv("NOT_CR
 # mirai daemon limits tests
 connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.5)
-  test_true(daemons(1, cleanup = FALSE, maxtasks = 2L, id = 125L))
+  test_true(daemons(1, cleanup = FALSE, maxtasks = 2L))
   test_true(daemons_set("default"))
   test_equal(mirai(1)[], mirai(1)[])
   m <- mirai(0L)
   Sys.sleep(1L)
   res <- status()
   test_zero(res$connections)
-  test_identical(res$events, c(125L, -125L))
   test_equal(res$mirai[["awaiting"]], 1L)
-  test_equal(launch_local(1, idletime = 5000L, walltime = 500L, id = 129L), 1L)
+  test_equal(launch_local(1, idletime = 5000L, walltime = 500L), 1L)
   test_zero(m[])
   Sys.sleep(1L)
   res <- status()
   test_zero(res$connections)
-  test_identical(res$events, c(129L, -129L))
   test_equal(info()[["cumulative"]], 2L)
   test_false(daemons(0))
   test_true(daemons(1, dispatcher = FALSE, maxtasks = 1L))
@@ -417,14 +427,13 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_true(daemons(2, seed = 1234L))
   test_equal(launch_local(), 1L)
   test_type("character", launch_remote())
-  Sys.sleep(0.5)
-  test_true(all(everywhere(TRUE)[.flat]))
+  test_class("mirai_map", everywhere(TRUE, .min = 3L))
   m <- mirai_map(1:12, rnorm)[]
   test_false(daemons(0))
   Sys.sleep(0.5)
-  test_true(daemons(4, dispatcher = "none", seed = 1234L, .compute = "gpu"))
+  test_true(daemons(4, dispatcher = FALSE, seed = 1234L, .compute = "gpu"))
   with_daemons("gpu", {
-    test_true(all(everywhere(TRUE)[.flat]))
+    test_class("mirai_map", everywhere(TRUE))
     n <- mirai_map(1:12, rnorm)[]
     test_false(daemons(NULL))
   })
@@ -438,6 +447,9 @@ connection && requireNamespace("otel", quietly = TRUE) && Sys.getenv("NOT_CRAN")
       unlockBinding("otel_tracing", ns)
       ns[["otel_tracing"]] <- TRUE
       lockBinding("otel_tracing", ns)
+      unlockBinding("otel_tracer", ns)
+      ns[["otel_tracer"]] <- otel::get_tracer(name = mirai:::otel_tracer_name)
+      lockBinding("otel_tracer", ns)
       url <- local_url()
       test_true(daemons(url = url, dispatcher = FALSE))
       mp <- mirai_map(1:3, rnorm)
@@ -450,38 +462,50 @@ connection && requireNamespace("otel", quietly = TRUE) && Sys.getenv("NOT_CRAN")
     error = function(cnd) NULL
   )
   length(record) && {
-    test_equal(length(record$traces), 13L)
-    test_equal(record$traces[[1L]]$name, "mirai::mirai")
-    test_equal(record$traces[[1L]]$kind, "client")
-    test_true(record$traces[[2L]]$attributes$mirai.id > record$traces[[1L]]$attributes$mirai.id)
-    test_equal(record$traces[[1L]]$parent, record$traces[[2L]]$parent)
-    test_equal(record$traces[[2L]]$links[[1L]]$span_id, record$traces[[3L]]$links[[1L]]$span_id)
-    test_equal(record$traces[[4L]]$name, "mirai::mirai_map")
-    test_equal(record$traces[[4L]]$kind, "internal")
-    test_equal(record$traces[[4L]]$links[[1L]]$span_id, record$traces[[3L]]$links[[1L]]$span_id)
-    test_equal(record$traces[[4L]]$span_id, record$traces[[2L]]$parent)
-    test_equal(record$traces[[5L]]$name, "mirai::mirai")
-    test_equal(record$traces[[5L]]$kind, "client")
-    test_equal(record$traces[[5L]]$parent, record$traces[[4L]]$parent)
-    test_equal(record$traces[[7L]]$name, "mirai::daemon->eval")
-    test_equal(record$traces[[7L]]$kind, "server")
-    test_equal(record$traces[[7L]]$parent, record$traces[[1L]]$span_id)
-    test_equal(record$traces[[10L]]$status, "error")
-    test_equal(record$traces[[10L]]$description, "miraiError")
-    test_equal(record$traces[[11L]]$status, "error")
-    test_equal(record$traces[[11L]]$description, "miraiInterrupt")
-    test_equal(record$traces[[12L]]$name, "mirai::daemon")
-    test_equal(record$traces[[12L]]$kind, "internal")
-    test_equal(record$traces[[12L]]$span_id, record$traces[[7L]]$links[[1L]]$span_id)
-    test_equal(record$traces[[13L]]$name, "mirai::daemons")
-    test_equal(record$traces[[13L]]$kind, "internal")
-    test_equal(record$traces[[13L]]$attributes$url, url)
-    test_equal(record$traces[[13L]]$attributes$n, 0L)
-    test_equal(record$traces[[13L]]$attributes$dispatcher, "false")
-    test_equal(record$traces[[13L]]$attributes$compute_profile, "default")
-    test_equal(record$traces[[13L]]$span_id, record$traces[[1L]]$links[[1L]]$span_id)
-    test_equal(record$traces[[13L]]$span_id, record$traces[[4L]]$links[[1L]]$span_id)
-    test_equal(record$traces[[13L]]$span_id, record$traces[[5L]]$links[[1L]]$span_id)
+    traces <- record$traces
+    test_equal(length(traces), 15L)
+    test_equal(traces[[1L]]$name, "daemons")
+    test_equal(traces[[1L]]$kind, "internal")
+    test_equal(traces[[1L]]$attributes$url, url)
+    test_equal(traces[[1L]]$attributes$n, 1L)
+    test_equal(traces[[1L]]$attributes$dispatcher, "false")
+    test_equal(traces[[1L]]$attributes$compute_profile, "default")
+    test_equal(traces[[2L]]$name, "mirai")
+    test_equal(traces[[2L]]$kind, "client")
+    test_equal(traces[[2L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+    test_true(traces[[3L]]$attributes$mirai.id > traces[[2L]]$attributes$mirai.id)
+    test_equal(traces[[3L]]$parent, traces[[2L]]$parent)
+    test_equal(traces[[4L]]$links[[1L]]$span_id, traces[[3L]]$links[[1L]]$span_id)
+    test_equal(traces[[5L]]$name, "mirai_map")
+    test_equal(traces[[5L]]$kind, "internal")
+    test_equal(traces[[5L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+    test_equal(traces[[5L]]$span_id, traces[[3L]]$parent)
+    test_equal(traces[[5L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+    test_equal(traces[[6L]]$name, "mirai")
+    test_equal(traces[[6L]]$kind, "client")
+    test_equal(traces[[6L]]$parent, record$traces[[5L]]$parent)
+    test_equal(traces[[6L]]$links[[1L]]$span_id, traces[[1L]]$span_id)
+    test_equal(traces[[8L]]$name, "daemon")
+    test_equal(traces[[8L]]$kind, "internal")
+    test_equal(traces[[8L]]$attributes$url, url)
+    test_equal(traces[[9L]]$name, "daemon->eval")
+    test_equal(traces[[9L]]$kind, "server")
+    test_equal(traces[[9L]]$parent, traces[[2L]]$span_id)
+    test_equal(traces[[9L]]$links[[1L]]$span_id, traces[[8L]]$span_id)
+    test_equal(traces[[12L]]$status, "error")
+    test_equal(traces[[12L]]$description, "miraiError")
+    test_equal(traces[[13L]]$status, "error")
+    test_equal(traces[[13L]]$description, "miraiInterrupt")
+    test_equal(traces[[14L]]$name, "daemon->end")
+    test_equal(traces[[14L]]$kind, "internal")
+    test_equal(traces[[14L]]$attributes$url, url)
+    test_equal(traces[[14L]]$links[[1L]]$span_id, traces[[8L]]$span_id)
+    test_equal(traces[[15L]]$name, "daemons->reset")
+    test_equal(traces[[15L]]$kind, "internal")
+    test_equal(traces[[15L]]$attributes$url, url)
+    test_equal(traces[[15L]]$attributes$n, 1L)
+    test_equal(traces[[15L]]$attributes$dispatcher, "false")
+    test_equal(traces[[15L]]$attributes$compute_profile, "default")
   }
 }
 test_false(daemons(0))
